class Task extends EventTarget {
  constructor(data){
    super()
    this.data = data
  }

  get done(){
    return this.data.done
  }

  set done(new_value){
    this.data.done = new_value
    this.dispatchEvent(new CustomEvent("change"))
  }

  get description(){
    return this.data.description
  }

  set description(new_value){
    this.data.description = new_value
    this.dispatchEvent(new CustomEvent("change"))
  }
}

let tasks = [
  new Task({done: true,  description: "Do Laundry" }),
  new Task({done: false, description: "Do Homework"}),
  new Task({done: false, description: "Do Chores"  }),
]

class TaskView {
  static template = `
    <input type="checkbox" name="done"/> 
    <task-description> -- </task-description>
  `

  constructor(task){
    this.task = task
    this.element = document.createElement("task-summary")
    this.element.innerHTML = this.constructor.template

    this.fields = {
      done: this.element.querySelector("[name=done]"),
    }

    this.update()

    this.task.addEventListener("change", () => this.update())

    this.fields.done.addEventListener("change", () => {
      this.task.done = this.fields.done.checked
    })

  }

  update() {
    this.fields.done.checked = this.task.done
    this.element.querySelector("task-description").innerText = this.task.description
  }
}

class DetailTaskView {

  constructor(element){
    this.element = element
    this.fields = {
      done: this.element.querySelector("[name=done]"),
      description: this.element.querySelector("[name=description]")
    }

    this.fields.description.addEventListener("input", () => {
      this.task.description = this.fields.description.value
    })

    this.fields.done.addEventListener("change", () => {
      this.task.done = this.fields.done.checked
    })
  }

  get task(){
    return this._task
  }

  set task(new_task){
    this._task = new_task
    this._task.addEventListener("change", () => this.update())
    this.update()
  }

  update() {
    this.fields.done.checked = this.task.done
    this.fields.description.value = this.task.description
  }
}

let detail_view = new DetailTaskView(document.querySelector("task-detail"))

let list = document.querySelector("task-list")

for (let task of tasks){
  let view = new TaskView(task)

  view.element.addEventListener("click", () => detail_view.task = task )

  list.appendChild(view.element)
}